typedef unsigned char BYTE;
typedef unsigned short WORD;

#define DESC_TYPE_DEVICE			0x01	
#define DESC_TYPE_CONFIG			0x02	
#define DESC_TYPE_STRING			0x03
#define DESC_TYPE_INTERFACE 		0x04	
#define DESC_TYPE_ENDPOINT			0x05

#pragma pack(1)

#define SIZEOF_DEVICE_DESC 0x12
typedef	struct _DEVICE_DESCRIPTOR
{
	BYTE	bLength;				// Length of this descriptor (12h bytes)
	BYTE	bDescriptorType;	  // Type code of this descriptor (01h)
	WORD	bcdUsb;					// Release of USB spec (0210h =	rev	2.10)
	BYTE	bDeviceClass;		//Device's base	class code
	BYTE	bDeviceSubClass;	  //Device's sub class code
	BYTE	bDeviceProtocol;	  //Device's protocol type code
	BYTE	bMaxPacketSize0;	  //End	point 0's max packet size (8/16/32/64)
	WORD	idVendor;			// Vendor ID for device
	WORD	idProduct;			  // Product ID	for	device
	WORD	bcdDevice;			  // Revision level	of device
	BYTE	iManufacturer;		  // Index of manufacturer name	string desc
	BYTE	iProduct;			// Index of	product	name string	desc
	BYTE	iSerialNumber;		  // Index of serial number	string desc
	BYTE	bNumConfigurations;	// Number of configurations	supported
}DEVICE_DESCRIPTOR;


#define SIZEOF_CONFIG_DESCRIPTOR 0x09
typedef struct _CONFIG_DESCRIPTOR
{
    BYTE	bLength;		          // Length of this descriptor (9h bytes)
    BYTE	bDescriptorType;	    // Type code of this descriptor (02h)
    WORD	wTotalLength; 	      // Size of this config desc plus all interface,
    //	endpoint, class, and vendor descriptors
    BYTE	bNumInterfaces; 	    // Number of interfaces in this config
    BYTE	bConfigurationValue;	// Value to use in SetConfiguration command
    BYTE	iConfiguration; 	    // Index of string desc describing this config
    BYTE	bmAttributes; 	      // See CFG_DESC_ATTR_xxx values below
    BYTE	MaxPower;   	        // Power used by this config in 2mA units
}CONFIG_DESCRIPTOR;


#define SIZEOF_INTERFACE_DESCRIPTOR 0x09
typedef struct _INTERFACE_DESCRIPTOR
{
    BYTE	bLength;						// Length of this descriptor (9h bytes)
    BYTE	bDescriptorType;		// Type code of this descriptor (04h)
    BYTE	bInterfaceNumber; 	// Zero based index of interface in the configuration
    BYTE	bAlternateSetting;	// Alternate setting number of this interface
    BYTE	bNumEndpoints;		  // Number of endpoints in this interface
    BYTE	bInterfaceClass;	  // Interface's base class code
    BYTE	bInterfaceSubClass; // Interface's sub class code
    BYTE	bInterfaceProtocol; // Interface's protocol type code
    BYTE	iInterface; 	      // Index of string desc describing this interface
}INTERFACE_DESCRIPTOR;


#define SIZEOF_ENDPOINT_DESCRIPTOR 0x07
typedef struct _ENDPOINT_DESCRIPTOR
{
    BYTE	bLength;					// Length of this descriptor (7h bytes)
    BYTE	bDescriptorType;	// Type code of this descriptor (05h)
    BYTE	bEndpointAddress; // See EP_DESC_ADDR_xxx values below
    BYTE	bmAttributes; 		// See EP_DESC_ATTR_xxx value below
    WORD	wMaxPacketSize; 	// Max packet size of endpoint
    BYTE	bInterval;				// Polling interval of endpoint in milliseconds
}ENDPOINT_DESCRIPTOR;

#pragma pack()

//
// Unicode string descriptor
//
#define TYPEDEF_UNICODE_STRING_DESC(typedef_name, str_len)  \
typedef struct typedef_name##_tag           \
{                                           \
    BYTE bLength;                           \
    BYTE bDescriptorType;                   \
    WORD bString[str_len];                  \
} typedef_name


#define USB_AUDIO_CLASS_CODE		0x01
#define USB_HID_CLASS_CODE			0x03